<?php

namespace Drupal\search_api_rabbit_hole\Plugin\search_api\processor;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\search_api\IndexInterface;
use Drupal\search_api\Plugin\PluginFormTrait;
use Drupal\search_api\Processor\ProcessorPluginBase;

/**
 * Excludes some Rabbit Holed entities from being indexed.
 *
 * @SearchApiProcessor(
 *   id = "indexing_rabbit_hole_filter",
 *   label = @Translation("Indexing Rabbit Hole Filter"),
 *   description = @Translation("Prevent some items from being indexed depending on rabbit hole settings."),
 *   stages = {
 *     "alter_items" = 0,
 *   },
 * )
 */
class IndexingRabbitHoleFilter extends ProcessorPluginBase implements PluginFormInterface {

  use PluginFormTrait;

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $configuration = parent::defaultConfiguration();

    $configuration += [
      'prevent_indexing_plugins' => [
        'access_denied',
        'page_not_found',
      ],
    ];

    return $configuration;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['prevent_indexing_plugins'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Rabbit Hole indexing preventing plugins'),
      '#description' => $this->t('Select which Rabbit Hole Plugins should prevent indexing of the content item.'),
      '#default_value' => $this->configuration['prevent_indexing_plugins'] ?? [],
      '#required' => TRUE,
      '#multiple' => TRUE,
    ];

    /** @var \Drupal\rabbit_hole\Plugin\RabbitHoleBehaviorPluginManager $rabbit_hole_plugin_manager */
    /* @phpstan-ignore-next-line */
    $rabbit_hole_plugin_manager = \Drupal::service('plugin.manager.rabbit_hole_behavior_plugin');
    $definitions = $rabbit_hole_plugin_manager->getDefinitions();

    // In case a definition no longer exists, clean it from default values.
    $form['prevent_indexing_plugins']['#default_value'] = array_intersect(
      $form['prevent_indexing_plugins']['#default_value'],
      array_keys($definitions),
    );
    foreach ($definitions as $id => $definition) {
      $form['prevent_indexing_plugins']['#options'][$id] = $definition['label'];
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function requiresReindexing(?array $old_settings = NULL, ?array $new_settings = NULL) {
    $old = $old_settings['prevent_indexing_plugins'] ?? [];
    $old = array_filter($old);
    $new = $new_settings['prevent_indexing_plugins'] ?? [];
    $new = array_filter($new);

    // If Rabbit Hole plugins selection changes, re-index is required.
    if (array_diff($old, $new)) {
      return TRUE;
    }

    return parent::requiresReindexing($old_settings, $new_settings);
  }

  /**
   * {@inheritdoc}
   */
  public static function supportsIndex(IndexInterface $index) {
    /** @var \Drupal\rabbit_hole\Plugin\RabbitHoleEntityPluginManager $rabbit_hole_entity_manager */
    $rabbit_hole_entity_manager = \Drupal::service('plugin.manager.rabbit_hole_entity_plugin');
    $rabbit_hole_entity_type_ids = $rabbit_hole_entity_manager->loadSupportedEntityTypes();

    foreach ($index->getDatasources() as $datasource) {
      $entity_type_id = $datasource->getEntityTypeId();
      if (!$entity_type_id) {
        continue;
      }

      if (in_array($entity_type_id, $rabbit_hole_entity_type_ids)) {
        return TRUE;
      }
    }
    return FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function alterIndexedItems(array &$items) {
    $prevent_indexing_plugins = $this->getConfiguration()['prevent_indexing_plugins'];
    $prevent_indexing_plugins = array_filter($prevent_indexing_plugins);
    foreach ($items as $item_id => $item) {

      // Check if the entity has Rabbit Hole settings stored.
      /** @var \Drupal\search_api\Item\ItemInterface $item */
      $entity = $item->getOriginalObject()->getValue();
      if (
        $entity instanceof ContentEntityInterface
        && $entity->hasField('rh_action')
      ) {

        /** @var \Drupal\rabbit_hole\BehaviorInvokerInterface $rabbit_hole_settings */
        /* @phpstan-ignore-next-line */
        $rabbit_hole_settings = \Drupal::service('rabbit_hole.behavior_invoker')->getRabbitHoleValuesForEntity($entity);
        if (in_array($rabbit_hole_settings['rh_action'], $prevent_indexing_plugins)) {
          unset($items[$item_id]);
        }
      }
    }
  }

}
